@@ -2,3 +2,5 @@ from .DISRwithMassive import *
 from .FCBF import *
 from .MultivariateFilter import MultivariateFilter
 from .measures import *
+from .TraceRatioFisher import TraceRatioFisher
+from .STIR import STIR
